from flake8_class_newline.__version__ import __version__

new_lines_on = {}

DOUBLE_QUOTE = '"'
SINGLE_QUOTE = "'"
CLASS = 'class'
METHOD = 'def '
DECORATOR = '@'


def new_line_checker(logical_line, line_number, filename):
    if filename not in new_lines_on:
        new_lines_on[filename] = []

    if logical_line.startswith(CLASS):
        new_lines_on[filename].append(line_number + 1)

    if line_number in new_lines_on[filename]:
        # if the line is not only whitespace and not empty
        double_quotes = logical_line.strip().startswith(DOUBLE_QUOTE * 3)
        quotes = logical_line.strip().startswith(SINGLE_QUOTE * 3)

        is_method = logical_line.strip().startswith(METHOD)
        is_decorator = logical_line.strip().startswith(DECORATOR)
        is_callable = is_method or is_decorator

        not_only_space = not logical_line.isspace() and logical_line
        not_is_docstring = not (quotes or double_quotes)

        if not_only_space and not_is_docstring and is_callable:
            # this means there's something on the line
            offset = line_number, 0
            yield offset, 'CNL100 Class definition does not have a new line.'


new_line_checker.name = 'new_line_checker'
new_line_checker.version = __version__



from flake8_class_newline import new_line_checker
from unittest import TestCase


class TestChecks(TestCase):

    def check_file(self, lines, filename):
        errors = []
        for line_number, line in enumerate(lines):
            errors.append(
                list(
                    new_line_checker(
                        logical_line=line,
                        line_number=line_number + 1,
                        filename=filename
                    )
                )
            )

        return errors

    def test_checks_class_new_lines(self):
        expected_errors = [
            [],
            [((2, 0), 'CNL100 Class definition does not have a new line.')],
            [],
            [],
            [],
            [],
            [],
            [((8, 0), 'CNL100 Class definition does not have a new line.')],
            [],
            [],
            [],
        ]

        an_error_file = [
            'class ClassWithoutANewLineDecorator(object):',
            '   @property',
            '   def aaasome_arg(self):',
            '      return "a_string"',
            '',
            '',
            'class ClassWithoutANewLineMethod(object):',
            '   def aaasome_arg(self):',
            '      return "a_string"',
            '',
            '',
        ]
        # This should produce no error.
        no_error_file = [
            'class ClassWithANewLineDecorator(object):',
            '',
            '   @property',
            '   def aaasome_arg(self):',
            '      return "a_string"',
            '',
            '',
            'class ClassWithANewLineMethod(object):',
            '',
            '   def aaasome_arg(self):',
            '      return "a_string"',
            '',
            '',
            'class ClassWithoutANewLineArgument(object):',
            '   an_arg = "a_value"',
            '',
            '',
            'class ClassWithoutANewLineArgument(object):',
            '   definetly_an_argument = "a_value"',
            '',
            '',
            'class ClassWithMultipleBases(AClassWithNewLine,'
            '                             ClassWithDoubleDocstring,'
            '                             object):',  # The same logical line
            "   ''' A docstring '''",
            '   some_arg = "a_string"',
            '',
            '',
            'class ClassWithDoubleDocstringShouldBeIgnored(object):',
            '""" A docstring """',
            '   some_arg = "a_string"',
            '',
            '',
            'class ClassWithSingleDocstringShouldBeIgnored(object):',
            "   ''' A docstring '''",
            '   some_arg = "a_string"',
            '',
            '',
            'class ClassWithClassMetaobject):',
            "   class Meta:",
            '      an_arg = "a_value"',
            '',
            '',
        ]

        an_error_file_errors = self.check_file(an_error_file, 'a_file')
        no_error_file_errors = self.check_file(no_error_file, 'another_file')

        self.assertEqual(expected_errors, an_error_file_errors)
        self.assertEqual(
            [[] for _ in no_error_file_errors], no_error_file_errors
        )
